"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createDomain = createDomain;
exports.domain = domain;
exports.rootDomain = rootDomain;

var _most = require("most");

var _mostSubject = require("most-subject");

var _index = require("./index.h");

var _config = require("./config");

function createDomain(store, domainName = '') {
  var redux = store;
  var config = redux.dispatch({
    type: _config.PING
  }).payload;

  var addPlainHandler = (name, fn) => {
    config.plain.set(name, fn);
  };

  var state$ = (0, _most.from)(redux).map(() => store.getState()).multicast();
  return DomainConstructor(domainName, redux.dispatch, store.getState, state$, new Map(), addPlainHandler);
}

function domain(domainName, dispatch, getState, state$) {
  return DomainConstructor(domainName, dispatch, getState, state$);
}

function rootDomain(domainName = '') {
  var addPlainHandler = (name, fn) => {
    config.plain.set(name, fn);
  };

  var redux;
  var reduxStore;
  var config = {
    plain: new Map()
  };

  function dispatch(data) {
    return redux.dispatch(data);
  }

  function getState() {
    return reduxStore.getState();
  }

  var state$ = (0, _mostSubject.async)();

  function register(store) {
    redux = reduxStore = store;
    var pong = redux.dispatch({
      type: _config.PING
    });

    if (pong) {
      var plain = pong.payload.plain;
      var oldPlain = config.plain;

      for (var [key, val] of oldPlain.entries()) {
        if (!plain.has(key)) {
          plain.set(key, val);
        }
      }

      config.plain = plain;
    } //$off


    (0, _most.from)(store).map(() => getState()).observe(state => state$.next(state));
  }

  var domain = DomainConstructor(domainName, dispatch, getState, state$, new Map(), addPlainHandler);
  domain.register = register;
  return domain;
}

var nextPayloadID = (0, _index.counter)();
var nextEventID = (0, _index.counter)();

function DomainConstructor(domainName, dispatch, getState, state$, events = new Map(), addPlainHandler = () => {}) {
  return {
    register(store) {
      console.warn(`Not implemented`);
    },

    effect(name) {
      return EffectConstructor(domainName, dispatch, getState, state$, events, name);
    },

    domain(name) {
      return DomainConstructor([domainName, name].join('/'), dispatch, getState, state$, events, addPlainHandler);
    },

    event(name) {
      return EventConstructor(domainName, dispatch, getState, state$, events, name);
    },

    typeConstant(name) {
      var action$ = (0, _mostSubject.async)();
      addPlainHandler(name, data => {
        action$.next(data);
      });
      var result = EventConstructor('', dispatch, getState, state$, events, name, action$);
      return result;
    }

  };
}

function EffectConstructor(domainName, dispatch, getState, state$, events, name) {
  var handlers = new Set();
  var eventID = nextEventID();
  var nextSeq = (0, _index.counter)();
  var action$ = (0, _mostSubject.async)();

  var getType = () => (0, _index.toTag)(domainName, name);

  handlers.add((payload, state) => action$.next(payload)); // handlers.add((payload, state) => state$.next(state))

  var using = never;

  function use(thunk) {
    using = thunk;
  }

  var create = payload => ({
    type: getType(),
    payload,
    meta: {
      index: nextPayloadID(),
      eventID,
      seq: nextSeq()
    }
  });

  var effect = params => {
    var run = () => {
      run = () => {};

      using(params).then(result => {
        var data = {
          params,
          result
        };
        dispatch(done(data));
        resolve(data);
      }, error => {
        var data = {
          params,
          error
        };
        dispatch(fail(data));
        reject(data);
      });
    };

    var runner = dispatchHook => {
      if (canDispatch) {
        var dispatcher = dispatchHook ? dispatchHook : dispatch;
        canDispatch = false;
        dispatcher(result);

        for (var handler of handlers) {
          handler(params, getState());
        } // action$.next(params)


        run();
      }
    };

    var resolve = val => {};

    var reject = val => {};

    var doneP = new Promise(rs => resolve = rs);
    var failP = new Promise(rs => reject = rs);
    var toSend = Promise.resolve(params);
    var result = create(params);
    var canDispatch = true;
    return Object.assign({}, result, {
      raw() {
        return result;
      },

      send(dispatchHook) {
        runner(dispatchHook);
        return toSend;
      },

      done() {
        runner();
        return doneP;
      },

      fail() {
        runner();
        return failP;
      },

      promise() {
        runner();
        return Promise.race([doneP, failP.then(data => Promise.reject(data))]);
      }

    });
  };

  var done = EventConstructor(domainName, dispatch, getState, state$, events, [name, 'done'].join(' '));
  var fail = EventConstructor(domainName, dispatch, getState, state$, events, [name, 'fail'].join(' '));

  function watch(fn) {
    handlers.add((params, state) => {
      var result = fn(params, state);
      safeDispatch(result, dispatch);
    });
  }

  function epic(handler) {
    var result = handler(action$, state$).multicast();
    result.observe(data => safeDispatch(data, dispatch));
    return result;
  }

  effect.use = use;
  effect.getType = getType;
  effect.done = done;
  effect.fail = fail;
  effect.watch = watch;
  effect.epic = epic;
  return effect;
}

function safeDispatch(data, dispatch) {
  if (typeof data === 'object' && data != null && typeof data.type === 'string') {
    dispatch(data);
  }
}

function EventConstructor(domainName, dispatch, getState, state$, events, name, action$ = (0, _mostSubject.async)()) {
  var eventID = nextEventID();
  var nextSeq = (0, _index.counter)();

  var getType = () => (0, _index.toTag)(domainName, name);

  var handlers = new Set();

  function watch(fn) {
    handlers.add(fn);
  }

  handlers.add((payload, state) => action$.next(payload)); // handlers.add((payload, state) => state$.next(state))

  function epic(handler) {
    var result = handler(action$, state$).multicast();
    result.observe(data => safeDispatch(data, dispatch));
    return result;
  }

  var create = payload => ({
    type: getType(),
    payload,
    meta: {
      index: nextPayloadID(),
      eventID,
      seq: nextSeq()
    }
  });

  var eventInstance = payload => {
    var toSend = Promise.resolve(payload);
    var result = create(payload);
    var canDispatch = true;
    return Object.assign({}, result, {
      raw() {
        return result;
      },

      send(dispatchHook) {
        if (canDispatch) {
          var dispatcher = dispatchHook ? dispatchHook : dispatch;
          canDispatch = false;
          dispatcher(result);

          for (var handler of handlers) {
            handler(payload, getState());
          } // action$.next(payload)

        }

        return toSend;
      }

    });
  };

  eventInstance.getType = getType; //$off

  eventInstance.toString = getType;
  eventInstance.watch = watch;
  eventInstance.epic = epic;
  events.set(getType(), eventInstance);
  return eventInstance;
}

function never(props) {
  console.warn(`


  Running an effect without implementation

`, props);
  return new Promise(() => {});
}
//# sourceMappingURL=effector.js.map